;; positioner for Pace MSS300 series,
;; compatible with other Pace models and clones,
;; not tested though.
;;
;; Veendam, 13. september 1997 23:12:03
;;
;; Author G. Zuideveld
;;
;;
;; Description of the signal coming from the reciever.
;;
;; The data is coming in a serial format from pin 12 of the
;; decoder scart, use pin 13 as ground. The signal is normal
;; low. Data consists of small (appr. 2 S) pulses of 5V, with a
;; a variable time inbetween. This variable time determines
;; whether a bit is 1 or 0. 1`s are presented as a pause of
;; 82 s, 0`s are 64 s(+/- 2s) . Data is coming with 8 bits,
;; so there are 9 pulses, containing one whole byte of data.
;; There are bytes with special meaning.
;; Byte FF indicates power-off, 00 seems to occur
;; when rubbish enters the IR-reciever of the receiver.
;; The pace supports up to 64 satellites (1-64).
;;
;; Actuator`s reed switch for positoining needs to be
;; connected with a 1k resistor for pullup and a 100n cap.
;; for debouncing the switch. The switch needs to be connected to
;; the INT1 pin. Some surge protect diodes are also recommended.
;;
;; V0.00 18-09-1997 MSS-serial port is working
;; V0.01 20-09-1997 Pulse counting positioner seems to work
;; V0.02 27-09-1997 I2C & EEPROM detect build in
;; V0.03 02-10-1997 Pulse counting debugged, I2C debugged
;; V0.04 07-10-1997 added most functions & dialog
;; V0.05 07-10-1997 ALL ?? functions seem to work & are implemented
;; V0.10 09-10-1997 First Beta version, ready for lifetest.
;; V0.11 10-10-1997 2nd Beta version, memory is mapped ok.
;; V0.12 11-10-1997 Changed IO-pins to meet PCB design.



STANDALONE  EQU     1           ;; determines way of compiling.
TEST        EQU     0           ;; give some extra debugging info

include "8051_include:sfrnames.i"
include "8051_include:ASCII.i"
include "8051_include:I2C.i"
IF (!STANDALONE) include "8051_include:monitor.i"


/* FOLD */ ;; DEFINITIONS
;; Serial port speed:


  ;;SerSpeed  EQU   243             /* Default speed serial port = 4800 Bd */
  SerSpeed  EQU   254     /* speed serial port = 31250 Bd */





;; register declarations:

    STACK   EQU     060H        ;; initial stack pointer !! only 32 deep !

    NEWMSS  EQU     02EH.0      ;; new MSS-byte flag

    MSSBITC EQU     040H        ;; current bit of MSS-port
    MSSBYTE EQU     041H        ;; contains new byte

    CPOSHI   EQU     042H       ;; currentposition Hibyte
    CPOSLO   EQU     043H

    LastSt  EQU     044H        ;; IC Last start condition if set

    ICADDR  EQU     045H        ;; address I2C chip

    I2ACK   EQU     02EH.1      ;; contains Ack-bit after a I2C read or write

    TMP     EQU     046H

    SATELLITE EQU   047H


    DPOSHI  EQU     048H        ;; desired position
    DPOSLO  EQU     049H

    POS_CHK EQU     02EH.2      ;; 1 = position ok
    NewPos  EQU     02EH.3      ;; position has changed, set by interrupt
                                ;; cleared by user program

    TEMPBIT EQU     02EH.7      ;; temp bit

;; ports:

    LED     EQU     P1.0

    POL     EQU     P1.4       ;; relay to steer actuators` direction
    MOT     EQU     P1.5       ;; actuators` motor on/off

    EAST    EQU     P1.2        ;; set east (chan -)
    WEST    EQU     P1.1        ;; set west (chan +)
    STORE   EQU     P1.3        ;; store current setting

    SCL     EQU     P1.7        ;; !! NOT standard !!
    SDA     EQU     P1.6        ;; hardware pins

    MSSDATA EQU     IE0         ;; Pace MSS serial port
    PULSE   EQU     IE1         ;; pulse coutn from actuator





;; BASE, declaration of program offset
if (STANDALONE) {

    BASE        EQU     00000H
    } else      {
    BASE        EQU     04100H
    }



;; #################################################################
;; #################################################################
;; #################################################################
;; #################################################################
;; #################################################################

/* FEND */
    ORG     BASE                ;; BASE Address

START:                          ;; reset vector
/* FOLD */
    AJMP        INIT


;; IE0 used for serial interface with MSS300. Pin IE0 is the
;; physical interface.
  ORG           BASE+0003H      ;; IE0
    AJMP        INT_IE0

;; timer0 is the timer, used to determine the pulsewidt in between
;; two low-pulses of the serial interface.
;; an interrupt will only occur when the timer overflows, which
;; indicates an error
  ORG           BASE+000BH      ;; TF0
    AJMP        INT_TF0

;; IE1 is the physical interface for counting the pulses coming from
;; the actuator, corresponding with movement of the actuator.
;; This interrupt keeps track of the current position.
  ORG           BASE+0013H      ;; IE1
    AJMP        INT_IE1


/* FOLD */
;; not used
  ORG           BASE+001BH      ;; TF1
    AJMP        INT_TF1

;; not used
  ORG           BASE+0023H      ;; RI & TI
    AJMP        INT_RITI
/* FEND */

  ORG           BASE+0040H      ;; skip all vectors

/* FEND */

INIT:
/* FOLD */                               ;; conditional code for monitor compatibility


;; conditional assembly, only needed when executing from monitor
if (!STANDALONE)    {

        MOV   DPTR,#INT_IE0
        MOV   A,#1
        MOV   COMMAND,#ccLINK
        LCALL MON

        MOV   DPTR,#INT_TF0
        MOV   A,#2
        MOV   COMMAND,#ccLINK
        LCALL MON

        MOV   DPTR,#INT_IE1
        MOV   A,#3
        MOV   COMMAND,#ccLINK
        LCALL MON

        MOV   DPTR,#INT_TF1
        MOV   A,#4
        MOV   COMMAND,#ccLINK
        LCALL MON

        MOV   DPTR,#INT_RITI
        MOV   A,#5
        MOV   COMMAND,#ccLINK
        LCALL MON
                    }

    if (STANDALONE) MOV         SP,#STACK       ;; init stack

    if (TEST)       ACALL       EEDump          ;; check EEPROM

    ACALL       SetSerial       ;; set baudrate

    ACALL       I2Init          ;; set I2C-bus

    MOV         DPTR,#txt_welkom
    ACALL       TxTxt

    ACALL       InitMSSPort     ;; setup serial interface with receiver

    ACALL       InitActuator    ;; first set actuator to position 0

    ACALL       WAIT
    ACALL       WAIT
    ACALL       WAIT
    ACALL       WAIT

    ACALL       I2Init          ;; set I2C-bus


/* FEND */



MAIN:                           ;; main dialog


    MOV         SATELLITE,#01       ;; initialize at SAT_1

 1:
        IF (TEST)        {
        ACALL   WAIT
        ACALL   SNAP
        ACALL   WAIT
                         }
  ACALL       Setpos          ;; set to the current satellite

    ACALL       WriteScr

 5: ACALL       CheckMss
    JNC         3f
      ACALL     GetMssByte
      JNZ       2f              ;; zero, receiver error
        SJMP    5b              ;; just ignore
 2:   CJNE      A,#0FFH,2f      ;; FF, reciever shut-off
        ACALL   InitActuator    ;; re-init actuator
        SJMP    1b              ;; and restore current position
 2:   DEC       A
        ANL     A,#63           ;; SAT 1-64 are valid
        INC     A
        MOV     SATELLITE,A
        ACALL   WriteScr
        SJMP    1b
 2:

 3: ACALL       CheckBut
    JNC         4f
      ACALL       HandleBut
      ACALL       WriteScr

 4: SJMP        5b







/* FOLD */ ;; HandleBut, handle the pressed buttons
HandleBut:

    JB      EAST,8f
        ;; EAST is pressed
        CLR     POL             ;; EAST
        ACALL   SWAIT           ;; wait
        CLR     MOT             ;; start MOTor
     1: JNB     EAST,1b         ;; wait until button is released
        SETB    MOT             ;; stop motor
        ACALL   WAIT
        SETB    POL
        SJMP    9f              ;; exit

 8: JB      WEST,8f
        ;; WEST is pressed
        ACALL   SWAIT
        CLR     MOT             ;; start MOTor
     1: JNB     WEST,1b         ;; wait until button is released
        SETB    MOT             ;; stop motor
        ACALL   WAIT
        SETB    POL
        SJMP    9f              ;; exit

 8: JB      STORE,8f
        ;; STORE button is pressed
        MOV     R4,#8           ;; test for .8 secs, to prevent unwanted store
     1: JB      STORE,9f        ;; button is released, exit
        ACALL   SWAIT           ;; short wait 100mS
        DJNZ    R4,1b

        CLR     LED             ;; ok save and flash LED
        ACALL   SaveSat         ;; save current position
        ACALL   WAIT
        SETB    LED

     1: JNB     STORE,1b        ;; wait until button is released

        SJMP    9f              ;; exit

 8:

 9: RET


/* FEND */

/* FOLD */ ;; Setpos set the actuator to the current position (satellite)
SetPos:
    ACALL       LoadSat     ;; load the position from EEPROM

    ACALL       Checkpos    ;; check if position should be altered
    MOV         TEMPBIT,C   ;; temp save C
    MOV         A,DPL
    JNZ         2f
    MOV         A,DPH
    JNZ         2f
    SJMP        9f          ;; exit, current position matches desired one


 2: MOV         C,TEMPBIT   ;; determine direction by C, stored before
    MOV         POL,C       ;; and set POLarity

    ACALL       SWAIT       ;; wait short


    CLR         POS_CHK     ;; position not ok
    CLR         MOT         ;; and start motor

 2: ACALL       WriteScr    ;; write positions & settings to terminal

    ACALL       CheckMss    ;; check for new data on port
    JC          8f          ;; exit if new data

    ACALL       CheckBut    ;; check if a button is pressed
    JC          8f

    MOV         A,CPOSLO    ;; check poslo
    ACALL       Wait        ;; wait
    CLR         C
    SUBB        A,CPOSLO
    JZ          9f          ;; position did not change ! error !?
                            ;; build in for deadlock when endstops are
                            ;; reached without the position being satisfied,
                            ;; eg empty EEPROM location.

    JNB         POS_CHK,2b  ;; position ok ?
    SJMP        9f          ;; exit

 8: SETB        MOT         ;; stop MOTor, button pressed or new
                            ;; sat selected

 9: ACALL       WAIT        ;; wait before
    SETB        POL         ;; resetten POLarity (MOTORs` mass delay)
    ACALL       WAIT        ;; wait

    RET                     ;; and exit

/* FEND */

/* FOLD */ ;; *********** interrupt handlers **********************

;; external interrupt #0, used for serial interface with PACE MSS300
/* FOLD */
INT_IE0:                        ;; interrupt server for IE0

    PUSH    PSW
    PUSH    ACC
    SETB    RS0                 ;; switch to bank#1
    CLR     RS1


    JB      TCON.4,1f           ;; test if timer0 was started
        ;; timer0 not started, reset and start timer0
        MOV     TL0,#00         ;; clear timer0
        SETB    TCON.4          ;; start TIMER0
        MOV     MSSBITC,#8      ;; set bitcounter
        SJMP    9f              ;; exit

 1: MOV     A,TL0               ;; save timer0
    MOV     TL0,#00             ;; reset timer0
    SUBB    A,#73               ;; subtract to determine 1 or 0
    CPL     C
    MOV     A,R0                ;; R0 contains subsequent bits
    RRC     A                   ;; shift-in carry
    MOV     R0,A                ;; save result in R0
    DJNZ    MSSBITC,9f          ;; test for last bit
        ;; last bit received, stop timer0 and send byte
        CLR     TCON.4          ;; stop TIMER0
        MOV     MSSBYTE,A       ;; save byte
        SETB    NEWMSS          ;; set newbyte flag
        SJMP    9f              ;; exit

 9: POP     ACC                 ;; restore registers
    POP     PSW                 ;; and PSW

    RETI
/* FEND */

;; Timer0 interrupt, used for stop timer0 on error with MSS300
/* FOLD */
;; serial interface.
INT_TF0:

    CLR     TCON.4              ;; stop timer0, read timeout
    RETI
/* FEND */

;; external interrrupt #1, used for counting pulses from actuator, to
;; determine current position. And to stop motor at valid position.
INT_IE1:

    PUSH    PSW
    PUSH    ACC
    PUSH    DPL
    PUSH    DPH

    SETB         NewPos     ;; position has changed


    SETB    C
    CLR     A

    JB   POL,1f
      ADDC  A,CPOSLO     ;; carry was set before and A cleared
      MOV   CPOSLO,A
      CLR   A
      ADDC  A,CPOSHI     ;; if carry set +1 else +0
      MOV   CPOSHI,A
      SJMP  9f

 1:   MOV   A,CPOSLO
      SUBB  A,#0        ;; carry was set before, so -1
      MOV   CPOSLO,A     ;; save
      MOV   A,CPOSHI
      SUBB  A,#0        ;; if carry set -1, else -0
      MOV   CPOSHI,A

 9: JB      POS_CHK,9f  ;; check and wait if valid position
        ACALL   Checkpos
        MOV     A,DPL   ;; check if result = 0
        JNZ     9f      ;; if not skip
        MOV     A,DPH
        JNZ     9f      ;; if not skip
          ;; DPOS and CPOS match, stop motor.
          SETB  POS_CHK ;; valid position
          SETB  MOT     ;; stop motor
          ;; (do not reset POL yet)


 9: POP     DPH
    POP     DPL
    POP     ACC
    POP     PSW

    RETI                      
/* FOLD */
;; interrupt timer1, not used, timer1 is used for normal serial
;; interface as a baud rate generator.
INT_TF1:
    RETI

;; serial port interrupts, not used
INT_RITI:
    RETI
/* FEND */
/* FEND */

/* FOLD */ ;; InitMSSPort, set up timers and interrupts for the MSS-port.

InitMSSPort:

    ;; set up timer0:

        MOV     TH0,#00         ;; every 256 s */
        MOV     TL0,#00         ;; one interrupt from TIMER0
        MOV     TMOD,#022H      ;; both COUNTers MODE2 2

        SETB    IE.1            ;; switch TIMER0 interrupt on
        ;;SETB  TCON.4          ;; start TIMER0


    ;; set up IE0:

        SETB    IT0             ;; make interrupt occur on down-going
                                ;; edge of pulse, herefore the
                                ;; input doesn't need to be inverted !

        SETB    EX0             ;; enable external interrupt 0
        SETB    IE.7            ;; interrupts on

    RET
/* FEND */

/* FOLD */ ;; InitActuator, set up interrupt for position counter and
;; init actuator
InitActuator:

        CLR     POS_CHK         ;; set valid position

                                ;; start interrupts
        SETB    IT1             ;; edge trigerred
        ;; should be edge-triggered, otherwise the system will
        ;; continue interrupting while IE1 is low

        SETB    EX1             ;; enable external interrupt 1

        SETB    POL             ;; clear polarity

        ACALL   WAIT

    1:  CLR     MOT             ;; start motor, successive to prevent
                                ;; interrupt from doing this
        MOV     A,CPOSLO        ;; load position
        ACALL   WAIT            ;; wait .5 secs
        CLR     C               ;; clear carry !
        SUBB    A,CPOSLO        ;; did position change ?
        JNZ     1b              ;; if position changed, do again

        CLR     A
        MOV     CPOSLO,A        ;; position = 0
        MOV     CPOSHI,A

        SETB    MOT
        ACALL   WAIT            ;; WAIT for motor to run out

        RET
/* FEND */

/* FOLD */ ;; GetMSSByte, wait for byte on MSS-port and return it in A
GetMSSByte:                     ;; wait for byte on MSS-port and return
                                ;; it in A
                                ;; serial port emulates MSS-port

 1: JB          NEWMSS,2f       ;; wait for byte on Pace MSS port
    JB          SCON.0,3f       ;; wait for character on serial port

    SJMP        1b


 2: MOV         A,MSSBYTE       ;; get byte from Pace MSS port and
    CLR         NEWMSS
    RET                         ;; return

 3: MOV     A,SBUF              ;; get byte from serial port
    CLR     SCON.0
    CLR     C
    SUBB    A,#'0'              ;; translate ASCII into HEX (only 1-9 are valid)
    RET                         ;; and return
/* FEND */

/* FOLD */ ;; checkmss, check if data is present at MSS port data -> C=1
CheckMss:
    CLR         C
    ORL         C,NEWMSS        ;; check data MSS port
    ORL         C,SCON.0        ;; check data serial port
    RET
/* FEND */

/* FOLD */ ;; checkbut check if any button is pressed
CheckBut:
    CLR         C
    ORL         C,/EAST
    ORL         C,/WEST
    ORL         C,/STORE
    RET
/* FEND */

/* FOLD */ ;; SaveSat, save current position sat in EEPROM

SaveSat:                        ;; save current position sat in EEPROM

    MOV         A,#ADD_PCF8582
    ACALL       I2Addr

    MOV         A,SATELLITE     ;; load sat number in a
    DEC         A               ;; sat= (1-64) calculate EEPROM
    CLR         C               ;; address
    RLC         A               ;; x2 to store 16-bit position
    ACALL       I2Write         ;; set address of EEPROM

    MOV         A,CPOSLO
    ACALL       I2Write         ;; save lo-byte
    MOV         A,CPOSHI
    ACALL       I2Write         ;; save hi-byte

    ACALL       I2Free

    ACALL       Wait            ;; wait for EEPROM to finish

    RET
/* FEND */

/* FOLD */ ;; LoadSat, load position from EEPROM and return in DPTR

LoadSat:

    MOV         A,#ADD_PCF8582
    ACALL       I2Addr

    MOV         A,SATELLITE     ;; load sat number in a
    DEC         A               ;; sat= (1-64)
    CLR         C
    RLC         A               ;; x2 to store 16-bit position
    ACALL       I2Write         ;; set address of EEPROM

    ACALL       I2Read          ;; load lo-byte
    MOV         DPOSLO,A
    ACALL       I2Read          ;; load hi-byte
    MOV         DPOSHI,A

    RET                         ;; return position into DPTR
/* FEND */

/* FOLD */ ;; Checkpos, 16-bit subtraction of current & desired position
Checkpos:

    CLR     C
    MOV     A,DPOSLO
    SUBB    A,CPOSLO
 1: MOV     DPL,A
    MOV     A,DPOSHI
    SUBB    A,CPOSHI
 1: MOV     DPH,A        ;; store result in DPTR
                         ;; carry says direction !
 9: RET
/* FEND */

/* FOLD */ ;; Wait, wait for .5 secs
Wait:
    PUSH    DPL                 ;; save dptr
    PUSH    DPH

    MOV     DPTR,#500
    ACALL   LTime

    POP     DPH
    POP     DPL
    RET
/* FEND */

/* FOLD */ ;; SWait, wait for .2 secs
SWait:
    PUSH    DPL                 ;; save dptr
    PUSH    DPH

    MOV     DPTR,#200
    ACALL   LTime

    POP     DPH
    POP     DPL
    RET
/* FEND */

/* FOLD */ ;; TxCRLF, transmit CR & LF
TxCRLF:

    MOV         A,#LF
    ACALL       TxChr
    MOV         A,#CR
    ACALL       TxChr

    RET
/* FEND */

/* FOLD */ ;; hexbcd, turn a hex byte into a two-digit bcd number

    HexBcd: PUSH    ACC         ;; save for later use
            MOV     B,#10       ;; set division quotient
            CLR     C
            DIV     AB          ;; /10
            SWAP    A
            MOV     TMP,A
            SWAP    A
            MOV     B,#10       ;;
            MUL     AB          ;; *10
            XCH     A,B         ;; xch
            POP     ACC         ;; reload A
            CLR     C           ;;
            SUBB    A,B         ;; A-B
            ORL     A,TMP       ;; OR with high-digit
            RET
/* FEND */

/* FOLD */ ;; EEDump, dump EEPROM`s contents
;;if (TEST)   {
EEDump:

    PUSH        ACC
    PUSH        B0R3
    PUSH        B0R4

    MOV         R3,#00          ;; 256 addresses
    MOV         R4,#0           ;; address counter
    MOV         A,#ADD_PCF8582
    ACALL       I2Addr

    CLR         A               ;; set EEPROM address
    ACALL       I2Write


 1: MOV         A,R4
    ANL         A,#0FH
    JNZ         2f
      MOV         A,R4
      ACALL       TxByt
      MOV         A,#SPC
      ACALL       TxChr


 2: MOV         A,#SPC
    ACALL       TxChr
    ACALL       I2Read          ;; load byte from EEPROM
    ACALL       TxByt           ;; and print it


    MOV         A,R3
    DEC         A
    ANL         A,#0FH
    JNZ         2f
      ACALL       TxCRLF

 2: INC         R4

    DJNZ        R3,1b

    POP         B0R4
    POP         B0R3
    POP         ACC

    RET

;;            }
/* FEND */

/* FOLD */ ;; WriteScr: ;; write status on screen/lcd
WriteScr:                       ;; write status on screen/lcd

;; ******************
;; *Sat:00 CPos:0000*
;; *                *
;; ******************

    IF (STANDALONE) {
                    MOV     A,#CLS
                    ACALL   TxChr
                    }
    MOV     A,#'S'
    Acall   TxChr
    MOV     A,#'a'
    Acall   TxChr
    MOV     A,#'t'
    Acall   TxChr
    MOV     A,#':'
    Acall   TxChr
    MOV     A,SATELLITE
    ACALL   HexBcd
    ACALL   TxByt
    MOV     A,#' '
    ACALL   TxChr
    MOV     A,#'C'
    ACALL   TxChr
    MOV     A,#'P'
    ACALL   TxChr
    MOV     A,#'o'
    ACALL   TxChr
    MOV     A,#'s'
    ACALL   TxChr
    MOV     A,#':'
    ACALL   TxChr
    MOV     A,CPOSHI
    ACALL   TxByt
    MOV     A,CPOSLO
    ACALL   TxByt
    IF (STANDALONE) ACALL   TxCRLF
    ELSE    {
            MOV     A,#' '
            ACALL   TxChr
            }


    MOV     A,#'D'
    ACALL   TxChr
    MOV     A,#'P'
    ACALL   TxChr
    MOV     A,#'o'
    ACALL   TxChr
    MOV     A,#'s'
    ACALL   TxChr
    MOV     A,#':'
    ACALL   TxChr

    MOV     A,DPOSHI
    ACALL   TxByt
    MOV     A,DPOSLO
    ACALL   TxByt

    MOV     A,#' '
    ACALL   TxChr

    ACALL   Checkpos
    ACALL   TxWrd

    IF (STANDALONE) ACALL   TxCRLF
    ELSE    {
            MOV     A,#CR
            ACALL   TxChr
            }

    RET
/* FEND */

/* FOLD */ ;; setserial, set baudrate

SetSerial:

        MOV     TMOD,#22h       /* Timer 1 & 2(MSS-port) is 8-bit autoreload timer     */

        MOV     PCON,#80H       /* SMOD = 1 */

        MOV     TH1,#SerSpeed
        MOV     TL1,#SerSpeed

        MOV     TCON,#40h       /* Start baudrate timer */
        MOV     SCON,#52h       /* 8bit UART, Enable reception, Xmit interrupt */

        CLR         SCON.0          ;; clear serial buffer

        RET



/* FEND */

/* FOLD */ ;; ******** Long Time delay = DPTR mSec ***********

LTime:  PUSH    ACC
        PUSH    B0R0
        INC     DPH

        MOV     B0R0,#245       /* Loop count (980 uSec - 12 MHz Xtal) */
LTime1: MOV     A,B0R0          /* 1          \         \ */
LTime2: NOP                     /* 1 \         |         | */
        NOP                     /* 1  > 4*B0R0  > +3*DPL  > +5*DPH */
        DJNZ    ACC,LTime2      /* 2 /         |         | */
        DJNZ    DPL,LTime1      /* 2          /          | */
        DJNZ    DPH,LTime1      /* 2                    / */
        POP     B0R0
        POP     ACC
        RET
/* FEND */

;; conditional code

/* FOLD */ ;; I2C routines
/*************************************** */
/*       IC routines by Theo Arends */
/*       [SCL freq. = (Xtal / 12) / 12 cycles kHz.] */
/*       Xtal = 11.0592 MHz then IC bus clock is 76.8 kHz. */
/*       Xtal = 12 MHz then IC bus clock is 100 kHz. */

/*       Usage start IC bus communication: */
/*       LCALL   I2Init          - Initialise IC bus */
/*       Usage read A from parallel IO: */
/*       MOV     A,#040h         - IO chip ID */
/*       LCALL   I2Addr          - Assign IC bus and set IC address */
/*       LCALL   I2Read          - Read data byte */
/*       LJMP    I2Free          - Free IC bus */
/*       Usage write A to parallel IO: */
/*       PUSH    ACC             - Write IO from A */
/*       MOV     A,#040h         - IO chip ID */
/*       LCALL   I2Addr          - Assign IC bus and set IC address */
/*       POP     ACC */
/*       LCALL   I2Write         - Write data byte */
/*       LJMP    I2Free          - Free IC bus */
/*       Usage read A from EEPROM: */
/*       MOV     A,#0A0h         - EEPROM chip ID */
/*       LCALL   I2Addr          - Assign IC bus and set IC address */
/*       MOV     A,#RegAddr */
/*       LCALL   I2Write         - Write address of data byte */
/*       LCALL   I2Read          - Read data byte */
/*       LJMP    I2Free          - Free IC bus */

I2Init: MOV     ICAddr,#1       /* Initialise IC bus */
        CLR     LastSt
I2Free: PUSH    ACC             /* Flush and free IC bus */
        MOV     A,ICAddr
        JNB     ACC.0,I2Free2   /* R/W bit */
        MOV     ICAddr,#0       /* Reset IC address */
        PUSH    B
        MOV     B,#8            /* Flush read */
I2Free1: LCALL   I2In
        RLC     A
        DJNZ    B,I2Free1
        POP     B
        SETB    C
        LCALL   I2Out           /* Put NAck */
I2Free2: JB      LastSt,I2Free3
        SETB    LastSt          /* Set last start flag */

/*                   _ */
/*       I2SDat:  ___ */
/*                 ___ */
/*       I2SClk:  _ */

        CLR     SDA   /* I2SDat IC bus free sequence */
        SETB    SCL   /* I2SClk */
        CLR     SDA   /* I2SDat */
        SETB    SDA   /* I2SDat */
        MOV     C,SCL /* I2SClk IC bus idle if C = 1 */
        ANL     C,SDA /* I2SDat */
I2Free3: POP     ACC
        RET

/*-------------------------------------- */

I2Addr: LCALL   I2Free          /* Start IC session with IC address from A */

/*                __ */
/*       I2SDat:    ___ */
/*                ____ */
/*       I2SClk:      _ */

        SETB    SCL /* I2SClk    IC bus start sequence */
        SETB    SDA /* I2SDat */
        CLR     SDA /* I2SDat */
        NOP                     /* <-- Critical for SAA1064 */
        CLR     SCL /* I2SClk */
        MOV     ICAddr,A
        LJMP    I2Writ2         /* Send IC bus IC address */

/*-------------------------------------- */

I2Write: PUSH    ACC             /* Write A to IC address ICAddr */
        MOV     A,ICAddr
        JNB     ACC.0,I2Writ1   /* R/W bit */
        CLR     ACC.0
        LCALL   I2Addr          /* Set change */
I2Writ1: CLR     LastSt
        POP     ACC
I2Writ2: PUSH    ACC             /* Send byte from A */
        PUSH    B
        MOV     B,#8


/*       I2SDat:  ___777777777777666666 ... 110000000000000 */
/*                     __          __           __ */
/*       I2SClk:  _____  __________  __ ... ____  _________ */

I2Writ3: RLC     A               /* 1   \ */
        LCALL   I2Out           /* 2+7  > 12 cycles period time */
        DJNZ    B,I2Writ3       /* 2   / */
        POP     B
        POP     ACC

/*                _ */
/*       I2SDat:   cccccc */
/*                  __ */
/*       I2SClk:  __  ___ */

I2In:    SETB   SDA   /* I2SDat         1  Get bit as Carry (Ack = 0) */
        NOP                     /*       1 */
        SETB    SCL   /* I2SClk         1 */
        MOV     C,SDA /*I2SDat          1 */
        CLR     SCL   /*I2SClk          1 */
        RET                     /*       2 = 7 */

/*-------------------------------------- */

I2Read:  MOV     A,ICAddr        /* Read A from IC address ICAddr */
        JB      ACC.0,I2Read1   /* R/W bit */
        SETB    ACC.0
        LCALL   I2Addr          /* Set change */
I2Read1: CLR     LastSt
        PUSH    B
        MOV     B,#8            /* Receive byte in A */


/*       I2SDat:  __777777777777666666 ... 110000000000000 */
/*                    __          __           __ */
/*       I2SClk:  ____  __________  __ ... ____  _________ */

I2Read2: LCALL   I2In            /* 2+7 \ */
        RLC     A               /* 1    > 12 cycles period time */
        DJNZ    B,I2Read2       /* 2   / */
        POP     B
        CLR     C


/*       I2SDat:  ccccccc */
/*                  __ */
/*       I2SClk:  __  ___ */

I2Out:   MOV    SDA,C        /* 2  Put Carry as bit (Ack) */
        SETB    SCL          /* 1 */
        NOP                     /* 1 */
        CLR     SCL /*I2SClk      1 */
        RET                     /* 2 = 7 */

/*************************************** */
/* FEND */


if (!STANDALONE) {
/* FOLD */ ;; monitor interface
SNAP:    MOV COMMAND,#ccSNAP  /* MONITOR set command */
         LJMP MON             /* call up monitor */

;;LTime:   MOV   COMMAND,#ccLLTime
         LJMP MON

;;I2Init:  MOV  COMMAND,#ccI2Init
         LJMP MON

;;I2Addr:  MOV  COMMAND,#ccI2Adr
         LJMP MON

;;I2Write: MOV  COMMAND,#ccI2Write
         LJMP MON

;;I2Read:  MOV  COMMAND,#ccI2Read
         LJMP MON

;;I2Free:  MOV  COMMAND,#ccI2Free
         LJMP MON



RxChr:   MOV  COMMAND,#ccRxChr   /* Fetch a character from the serial port */
         LJMP MON

TxChr:   MOV  COMMAND,#ccTxChr   /* Xmit character in A from the serial port */
         LJMP MON

TxTxt:   MOV  COMMAND,#ccTxTxt    /* MONITOR set command */
         LJMP MON                /* jump to MONITOR (RET from there) */

TxByt:   MOV  COMMAND,#ccTxByt   /* MONITOR set command */
         LJMP MON                /* jump to MONITOR (RET from there) */

TxWrd:   MOV  COMMAND,#ccTxWrd   /* MONITOR set command */
         LJMP MON                /* jump to MONITOR (RET from there) */





/* FEND */
                 }

IF (STANDALONE)  {
/* FOLD */ ;; STANDALONE routines

TxTxt:  CLR     A               /* Send text from program store at DPTR until 0 */
        MOVC    A,@A+DPTR
        JZ      TxTxt1
        LCALL   TxChr
        INC     DPTR
        SJMP    TxTxt
TxTxt1: RET


TxWrd:  PUSH    ACC
        MOV     A,DPH
        LCALL   TxByt
        MOV     A,DPL
        LCALL   TxByt
        POP     ACC
        RET


TxByt:  PUSH    ACC             /* Send BYTE hexadecimal , destroy ACC only */
        SWAP    A
        LCALL   TxNibl
        POP     ACC
TxNibl: ANL     A,#0Fh
        ADD     A,#'0'
        CJNE    A,#'9'+1,$+3
        JC      TxChr
        ADD     A,#7
TxChr: JNB     SCON.1,TxChr    /* Must not destroy any reg */
        CLR     SCON.1
        MOV     SBUF,A
        CJNE    A,#Lf,TxChr1
TxWait: PUSH    DPL
        PUSH    DPH
        MOV     DPTR,#2
        LCALL   LTime           /* Wait 25 mSec for slow scrolling terminals */
        POP     DPH
        POP     DPL
TxChr1: RET


RxChr:  JNB     SCON.0,RxChr    ;; Get character from serial port and return in A
        MOV     A,SBUF
        CLR     SCON.0
        RET

/* FEND */
                 }

/* FOLD */ ;; Texts
TXT_WELKOM: DB CLS,"GSP300 by Gerard",LF,CR
            DB     "GRADTJE@DDS.NL  ",LF,CR,0


TXT_POS:    DB  CR,"Current position: ",0

;;TXT_NEEPROM: DB "No EEPROM present, saving in RAM.",LF,CR,0
;;TXT_EEPROM: DB  "EEPROM present, saving in EEPROM.",LF,CR,0
/* FEND */


    END

